Package: EGSEA
Title: Ensemble of Gene Set Enrichment Analyses
Version: 1.40.0
Authors@R: 
        c(person(given="Monther", family="Alhamdoosh", role=c("aut", "cre"),
                email="m.hamdoosh@gmail.com"),
          person(given="Luyi", family="Tian", role=c("aut")),
          person(given="Milica", family="Ng", role=c("aut")),
          person("Matthew", "Ritchie",role=c("ctb"),
                    email="mritchie@wehi.edu.au"))
Description: This package implements the Ensemble of Gene Set Enrichment
    Analyses (EGSEA) method for gene set testing. EGSEA algorithm utilizes 
    the analysis results of twelve prominent GSE algorithms in the literature 
    to calculate collective significance scores for each gene set.
biocViews: ImmunoOncology, DifferentialExpression, GO, GeneExpression,
        GeneSetEnrichment, Genetics, Microarray, MultipleComparison,
        OneChannel, Pathways, RNASeq, Sequencing, Software,
        SystemsBiology, TwoChannel,Metabolomics, Proteomics, KEGG,
        GraphAndNetwork, GeneSignaling, GeneTarget, NetworkEnrichment,
        Network, Classification
Depends: R (>= 4.3.0), Biobase, gage (>= 2.14.4), AnnotationDbi, topGO
        (>= 2.16.0), pathview (>= 1.4.2)
Imports: PADOG (>= 1.6.0), GSVA (>= 1.12.0), globaltest (>= 5.18.0),
        limma (>= 3.20.9), edgeR (>= 3.6.8), HTMLUtils (>= 0.1.5),
        hwriter (>= 1.2.2), gplots (>= 2.14.2), ggplot2 (>= 1.0.0),
        safe (>= 3.4.0), stringi (>= 0.5.0), parallel, stats, metap,
        grDevices, graphics, utils, org.Hs.eg.db, org.Mm.eg.db,
        org.Rn.eg.db, RColorBrewer, methods, EGSEAdata (>= 1.3.1),
        htmlwidgets, plotly, DT
License: GPL-3
LazyLoad: yes
NeedsCompilation: no
Encoding: UTF-8
Suggests: BiocStyle, knitr, testthat
VignetteBuilder: knitr
RoxygenNote: 7.2.3
git_url: https://git.bioconductor.org/packages/EGSEA
git_branch: RELEASE_3_23
git_last_commit: 1fc4087
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
Packaged: 2026-04-28 22:05:35 UTC; biocbuild
Author: Monther Alhamdoosh [aut, cre],
  Luyi Tian [aut],
  Milica Ng [aut],
  Matthew Ritchie [ctb]
Maintainer: Monther Alhamdoosh <m.hamdoosh@gmail.com>
Built: R 4.6.0; ; 2026-04-29 04:41:37 UTC; unix
