# Generated by roxygen2: do not edit by hand

export(buildCustomIdx)
export(buildGMTIdx)
export(buildGeneSetDBIdx)
export(buildIdx)
export(buildKEGGIdx)
export(buildMSigDBIdx)
export(egsea)
export(egsea.base)
export(egsea.cnt)
export(egsea.combine)
export(egsea.logo)
export(egsea.ma)
export(egsea.ora)
export(egsea.sort)
export(generateReport)
export(getSetByID)
export(getSetByName)
export(getSetScores)
export(getlimmaResults)
export(limmaTopTable)
export(plotBars)
export(plotGOGraph)
export(plotHeatmap)
export(plotMethods)
export(plotPathway)
export(plotSummary)
export(plotSummaryHeatmap)
export(showSetByID)
export(showSetByName)
export(topSets)
exportClasses(EGSEAResults)
exportClasses(GSCollectionIndex)
exportMethods("$")
exportMethods(show)
exportMethods(summary)
import(Biobase)
import(EGSEAdata)
import(HTMLUtils)
import(ggplot2)
import(gplots)
import(hwriter)
import(pathview)
import(stringi)
import(topGO)
importClassesFrom(limma,EList)
importClassesFrom(limma,EListRaw)
importClassesFrom(limma,MArrayLM)
importFrom(AnnotationDbi,Ontology)
importFrom(DT,datatable)
importFrom(GSVA,gsva)
importFrom(PADOG,padog)
importFrom(RColorBrewer,brewer.pal)
importFrom(edgeR,DGEList)
importFrom(edgeR,calcNormFactors)
importFrom(gage,gage)
importFrom(gage,kegg.gsets)
importFrom(globaltest,gt)
importFrom(globaltest,gt.options)
importFrom(globaltest,result)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(grDevices,png)
importFrom(graphics,abline)
importFrom(graphics,legend)
importFrom(graphics,par)
importFrom(htmlwidgets,saveWidget)
importFrom(limma,camera)
importFrom(limma,contrasts.fit)
importFrom(limma,eBayes)
importFrom(limma,fry)
importFrom(limma,ids2indices)
importFrom(limma,lmFit)
importFrom(limma,mroast)
importFrom(limma,neqc)
importFrom(limma,plotMDS)
importFrom(limma,topTable)
importFrom(limma,voom)
importFrom(metap,logitp)
importFrom(metap,meanp)
importFrom(metap,sumlog)
importFrom(metap,sump)
importFrom(metap,sumz)
importFrom(metap,votep)
importFrom(metap,wilkinsonp)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,slot)
importFrom(org.Hs.eg.db,org.Hs.egGO2ALLEGS)
importFrom(org.Mm.eg.db,org.Mm.egGO2ALLEGS)
importFrom(org.Rn.eg.db,org.Rn.egGO2ALLEGS)
importFrom(parallel,detectCores)
importFrom(parallel,mclapply)
importFrom(plotly,ggplotly)
importFrom(safe,getCmatrix)
importFrom(safe,safe)
importFrom(safe,safe.toptable)
importFrom(stats,IQR)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,p.adjust)
importFrom(stats,pchisq)
importFrom(stats,phyper)
importFrom(stats,quantile)
importFrom(stats,var)
importFrom(utils,browseURL)
importFrom(utils,capture.output)
importFrom(utils,data)
importFrom(utils,packageVersion)
importFrom(utils,timestamp)
importFrom(utils,write.csv)
importFrom(utils,write.table)
